@@ -1,9 +1,12 @@
 from django.db import models
-from django.db.models import UniqueConstraint, Subquery
+from django.db.models import UniqueConstraint, Subquery, OuterRef, Count, \
+    Avg, StdDev, Max, Min
+from django.db.models.functions import Coalesce
 from django.contrib.auth.models import User
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes.fields import GenericForeignKey, \
     GenericRelation
+from django.urls import reverse
 from django.utils.safestring import mark_safe
 from django.utils.translation import gettext_lazy as _
 
@@ -85,6 +88,12 @@ class Presentation(models.Model):
     def __str__(self):
         return self.name
     
+    def get_absolute_url(self):
+        form = self.form
+        args = {'program_slug': form.program.slug, 'form_slug': form.slug,
+                'presentation': self.id}
+        return reverse('plugins:reviewpanel:presentation_admin', kwargs=args)
+    
     def inputs_section(self):
         if self.no_input: return None
         return 'inputs'
@@ -191,6 +200,64 @@ class Input(RankedModel):
         return Input.objects.filter(form=self.form)
 
 
+class Metric(models.Model):
+    class Meta:
+        constraints = [
+            UniqueConstraint(fields=['input', 'name'],
+                             name='unique_metric_name')
+        ]
+        ordering = ['input', 'position', 'pk']
+    
+    class MetricType(models.TextChoices):
+        COUNT = 'count', _('count')
+        AVG = 'avg', _('average')
+        STDDEV = 'stddev', _('standard deviation')
+        MAX = 'max', _('maximum')
+        MIN = 'min', _('minimum')
+    
+    input = models.ForeignKey(Input, models.CASCADE,
+                              related_name='inputs', related_query_name='input')
+    name = models.SlugField(max_length=20, allow_unicode=True)
+    type = models.CharField(max_length=16, choices=MetricType.choices,
+                            default=MetricType.COUNT)
+    cohort = models.ForeignKey('Cohort', models.CASCADE, null=True, blank=True,
+                               related_name='+')
+    count_value = models.PositiveIntegerField(null=True, blank=True)
+    count_is_divisor = models.BooleanField(default=False)
+    boolean_invert = models.BooleanField(default=False)
+    position = models.PositiveIntegerField(default=1)
+    admin_enabled = models.BooleanField(default=True)
+    panelist_enabled = models.BooleanField(default=False)
+    
+    def __str__(self):
+        return self.name
+    
+    def aggregate(self):
+        if self.type == self.MetricType.COUNT: return Count
+        elif self.type == self.MetricType.AVG: return Avg
+        elif self.type == self.MetricType.STDDEV: return StdDev
+        elif self.type == self.MetricType.MAX: return Max
+        elif self.type == self.MetricType.MIN: return Min
+    
+    def annotation(self, queryset, **kwargs):
+        qs = queryset.filter(input=self.input,
+                             **{ k: OuterRef(kwargs[k]) for k in kwargs })
+        if self.cohort: qs = qs.filter(cohort=self.cohort)
+        
+        if self.type == self.MetricType.COUNT and self.count_value is not None:
+            qs = qs.filter(value=self.count_value)
+        elif self.input.type != Input.InputType.BOOLEAN:
+            qs = qs.filter(value__gt=0)
+        
+        qs = qs.values(*kwargs.keys())
+        metric_subq = qs.annotate(v=self.aggregate()('value')).values('v')
+        expr = Subquery(metric_subq)
+        
+        if self.type == self.MetricType.COUNT: expr = Coalesce(expr, 0)
+        # TODO boolean invert
+        return expr
+
+
 class Panel(models.Model):
     class Meta:
         constraints = [
@@ -255,7 +322,7 @@ class Cohort(models.Model):
 class CohortMember(models.Model):
     cohort = models.ForeignKey(Cohort, models.CASCADE)
     content_type = models.ForeignKey(ContentType, models.CASCADE)
-    object_id = models.UUIDField()
+    object_id = models.UUIDField(db_index=True)
     member = GenericForeignKey()
     
     def __str__(self):
@@ -285,7 +352,7 @@ class Score(models.Model):
     form = models.ForeignKey(Form, models.CASCADE, related_name='scores',
                              related_query_name='score')
     content_type = models.ForeignKey(ContentType, models.CASCADE)
-    object_id = models.UUIDField()
+    object_id = models.UUIDField(db_index=True)
     submission = GenericForeignKey()
     cohort = models.ForeignKey(Cohort, models.CASCADE, related_name='scores',
                                related_query_name='score')
