@@ -13,7 +13,8 @@ def dereference_block(context, ref, submission):
     if not block_name: return ''
     block = context['blocks'][block_name]
     
-    if block.block_type() == 'custom': return getattr(submission, block_name)
+    if block.block_type() == 'custom':
+        return getattr(submission, block_name) or ''
     elif block.block_type() == 'stock':
         stock = block.stock
         values = { n: getattr(submission, stock.field_name(n))
@@ -38,3 +39,12 @@ def underscore(obj, name):
     attr = getattr(obj, '_' + name)
     if callable(attr): return attr()
     return attr
+
+@register.filter
+def language_label(lang):
+    if lang == 'und': return 'unknown language'
+    return lang # TODO
+
+@register.filter
+def subtitle_url(item, lang):
+    return item._artifact_url('subtitles_' + lang)
